package ru.csasq.cit_is_bot.ui.screens

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.ExtendedFloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.snapshotFlow
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.buildAnnotatedString
import androidx.navigation.NavController
import cit_is_bot.composeapp.generated.resources.Res
import cit_is_bot.composeapp.generated.resources.baseline_add_24
import org.jetbrains.compose.resources.painterResource
import ru.csasq.cit_is_bot.ui.components.SwitchButton
import ru.csasq.cit_is_bot.ui.dialogs.ScenarioDialog

data class Scenario(
    val id: Int? = null,
    val name: String,
    val time: String,
    val messageNumber: String,
    val isEnabled: Boolean,
)

val scenarioList = listOf(
    Scenario(1, "Текущее состояние сотрудников", "0 11 * * 1-5", "1 сообщение", true),
    Scenario(2, "Планы на обед", "45 11 * * 1-5", "2 сообщения", false),
)

@Composable
fun ScenariosScreen(
    navController: NavController,
    paddingValues: PaddingValues,
    floatingActionButtonState: MutableState<(@Composable () -> Unit)?>,
) {
    val verticalScrollState = rememberScrollState()
    val floatingActionButtonExpandedState = remember {
        mutableStateOf(true)
    }
    LaunchedEffect(verticalScrollState) {
        var previousValue = 0
        snapshotFlow {
            verticalScrollState.value
        }.collect {
            floatingActionButtonExpandedState.value = it <= previousValue
            previousValue = it
        }
    }
    Column(
        modifier = Modifier
            .padding(
                paddingValues = paddingValues,
            )
            .fillMaxSize()
            .verticalScroll(
                state = verticalScrollState,
            ),
    ) {
        val scenarioDialogState = remember {
            mutableStateOf<Scenario?>(null)
        }
        floatingActionButtonState.value = @Composable {
            ExtendedFloatingActionButton(
                text = {
                    Text(
                        text = "Добавить сценарий",
                    )
                },
                icon = {
                    Icon(
                        painter = painterResource(Res.drawable.baseline_add_24),
                        contentDescription = null,
                    )
                },
                onClick = {
                    scenarioDialogState.value = Scenario(
                        name = "",
                        time = "",
                        messageNumber = "",
                        isEnabled = true,
                    )
                },
                expanded = floatingActionButtonExpandedState.value,
            )
        }
        scenarioList.forEach {
            val scenarioSwitchState = remember {
                mutableStateOf(it.isEnabled)
            }
            val captionAnnotatedString = buildAnnotatedString {
                append(it.time)
                append(" \u2022 ")
                append(it.messageNumber)
            }
            SwitchButton(
                title = it.name,
                caption = captionAnnotatedString.text,
                switchState = scenarioSwitchState,
                onCheckedChange = { value ->
                    scenarioSwitchState.value = value
                },
                onClick = {
                    scenarioDialogState.value = it
                },
            )
        }

        scenarioDialogState.value?.let {
            ScenarioDialog(
                scenarioDialogState = scenarioDialogState,
            )
        }
    }
}